/***jacos.h*********************************************/
#ifndef _JACOS_H
#define _JACOS_H

#ifndef TRUE
  #define TRUE  1
#endif
#ifndef FALSE
  #define FALSE 0
#endif

#define NORMAL  1
#define SMALL   2
#define COMPACT 4

#if defined(_PIC14) || defined(_PIC12)
  #define _PIC_
  #define _HT_
  #include <pic.h>
  #include <ja_libr.h>
  #if defined _PIC14
    #include <prt_p16.h>
  #else
    #include <prt_p12.h>
  #endif
  #include <jamain.h>
#elif defined _PIC18
  #define _PIC18_
  #define _HT_
  #include <pic18.h>
  #include <ja_libr.h>
  #include <prt_p18.h>
#elif defined HI_TECH_C // _DSPICC_
  #define _DSPIC_
  #define _HT_
  #include <dspic.h>
  #include <ja_libr.h>
  #include <prt_hdsp.h>
  #define OST_BITS unsigned
#elif defined dsPIC30
  #define _DSPIC_
  #define _MC_
  #include <p30fxxxx.h>
  #include <ja_libr.h>
  #include <prt_d30.h>
  #define OST_BITS OST_U8
#elif defined __ICCDSPIC__
  #define _DSPIC_
  #define _IAR_
  #include <ja_libr.h>
  #include <prt_idsp.h>
  #define OST_BITS OST_U8
#elif defined __18CXX
  #define _PIC18_
  #define _MC_
  #include <p18cxxx.h>
  #include <ja_libr.h>
  #include <prt_mcc.h>
#elif defined __ICCPIC18__
  #define _PIC18_
  #define _IAR_
  #include <prt_i18.h>
#elif defined __ICCAVR__
  #define _AVR_
  #define _IAR_
  #include <ioavr.h>
  #include <ja_libr.h>
  #include <prt_avr.h>
#elif defined __GNUC__
  #define _AVR_
  #define _GCC_
  #include <avr/io.h>
  #include <ja_libr.h>
  #include <prt_gavr.h>
#elif defined __ICC430__
  #define _MSP_
  #define _IAR_
  #include <ja_libr.h>
  #include <prt_i430.h>
#elif defined __CODEVISIONAVR__
  #define _AVR_
  #define _CV_
  #include <ja_libr.h>
  #include "\jacos\incl\prt_cva.h"
#elif defined __CX51__ || defined __C51__
  #define _51_
  #define _KEIL_
  #include <reg51.h>
  #include <ja_libr.h>
  #include <prt_kc51.h>
#elif defined __ICCARM__
  #define _ARM_
  #define _IAR_
  #include <ja_libr.h>
  #include <prt_arm.h>
#else
  #error compiler is not supported
#endif

#define OSS_READY_Q       0x00
#define OSS_TIMER         0x01
#define OSS_EVENT         0x02
#define OSS_IDLE          0x04
#define OSS_TIMEOUT_STAT  0x08
#define OSS_TIMER_Q       0x10
#define OSS_EVENT_Q       0x20
#define OSS_STOPPED       0x40
#define OSS_TIMEOUT_TASK  0x80
#define OS_STS_WAIT_MASK  (OSS_TIMER_Q + OSS_EVENT_Q + OSS_STOPPED)
#define OS_STS_PRIO_MASK  0x0F

#define OS_ERROR    TRUE
#define OS_NOERR    FALSE
#define OS_WAIT     TRUE
#define OS_NOWAIT   FALSE

#undef  NULL
#define NULL  0

#if defined(OS_MAIN)
  #define OS_EXTERN
#else
  #define OS_EXTERN extern
#endif

#ifndef OS_CVALIF
  #define OS_CVALIF
#endif

#ifndef OS_CALL_COND
  #define OS_CALL_COND extern
#endif

#ifndef OS_VAR_LOC
  #define OS_VAR_LOC
#endif

#ifndef OST_U8
  #define OST_U8  unsigned char
#endif
#ifndef OST_S8
  #define OST_S8  signed char
#endif
#ifndef OST_U16
  #define OST_U16 unsigned short
#endif
#ifndef OST_S16
  #define OST_S16 signed short
#endif
#ifndef OST_U32
  #define OST_U32 unsigned long
#endif
#ifndef OST_S32
  #define OST_S32 signed long
#endif
#ifndef OST_U64
  #define OST_U64 unsigned long long
#endif

#define OST_ERR_MSG   OST_U8
#define OST_WAIT_MSG  OST_U8

#ifndef OST_BITS
  #define OST_BITS OST_U8
#endif

#ifdef OS_USE_RESUME
  #undef  OS_USE_STOP
  #define OS_USE_STOP
  #undef  OS_USE_TASK_STATUS
  #define OS_USE_TASK_STATUS
#endif

#ifndef OST_WORKSPACE
  #undef  OS_USE_WORKSPACE
#else
  #define OS_USE_WORKSPACE
#endif

#ifndef OST_ADDRESS
  #define OST_ADDRESS OST_FUNC
#endif

#ifndef OS_Tree_Rest
  #define OS_Tree_Rest(a)
#endif

#ifndef OSM_RTN_LOCKED
  #define OSM_RTN_LOCKED
#endif

#ifndef OSM_RTN_UNLOCKED
  #define OSM_RTN_UNLOCKED
#endif

#if !defined(OS_USE_PRIO) && (OS_MAX_PRIO >= 1)
  #define OS_USE_PRIO
#endif

#if defined(OS_USE_CSEM) || defined(OS_USE_BSEM)\
|| defined(OS_USE_MSG) || defined(OS_USE_FLAGS)
  #define OS_USE_EVENTS
#endif

#if OS_DELAY_SIZE == 0
  #undef OS_USE_DELAY
  #define OST_TIMER OST_U8
#elif OS_DELAY_SIZE == 1
  #define OS_USE_DELAY
  #define OST_TIMER OST_U8
#elif OS_DELAY_SIZE == 2
  #define OS_USE_DELAY
  #define OST_TIMER OST_U16
#elif OS_DELAY_SIZE == 4
  #define OS_USE_DELAY
  #define OST_TIMER OST_U32
#else
  #error "OS_DELAY_SIZE is not correct"
#endif

#ifndef OS_MSG_CVALIF
  #define OS_MSG_CVALIF
#endif

#if   OS_SEM_SIZE == 0
  #undef OS_USE_CSEM
  #undef OS_USE_BSEM
  #define OST_SEM OST_U8
#elif OS_SEM_SIZE == 1
  #define OST_SEM OST_U8
#elif OS_SEM_SIZE == 2
  #define OST_SEM OST_U16
#elif OS_SEM_SIZE == 4
  #define OST_SEM OST_U32
#else
  #error "OS_SEM_SIZE is not correct"
#endif

#if   OS_FLAGS_SIZE == 0
  #undef OS_USE_FLAGS
  #define OST_FLG OST_U8
#elif OS_FLAGS_SIZE == 1
  #define OST_FLG OST_U8
#elif OS_FLAGS_SIZE == 2
  #define OST_FLG OST_U16
#else
  #error "OS_FLAGS_SIZE is not correct"
#endif

#ifndef OS_Dis_Int
  #error "Interrupt disable is not defined ( OS_Dis_Int() )"
#endif

#ifndef OS_Ena_Int
  #error "Interrupt enable is not defined ( OS_Ena_Int() )"
#endif

#ifdef OS_PROTECT_ENABLE
  #define OS_Enter_Critical() OS_Dis_Int()
  #define OS_Leave_Critical() OS_Ena_Int()
#else
  #define OS_Enter_Critical()
  #define OS_Leave_Critical()
#endif

#ifndef OS_Pause_Critical
  #define OS_Pause_Critical()
#endif

/*
MSG
*/
#ifdef OS_USE_MSG_I
  #define OS_Post_MsgI(a, b) OS_Post_Msg_I(a, b)

  #define OS_Post_MsgR(a, b, c)\
  {\
    OS_Enter_Critical();\
    *(c) = OS_Post_Msg_I(a, b);\
    OS_Leave_Critical();\
  }

  #define OS_Post_Msg(a, b)\
  {\
    OS_Enter_Critical();\
    (void)OS_Post_Msg_I(a, b);\
    OS_Leave_Critical();\
  }
#else
  #define OS_Post_MsgR(a, b, c) *(c) = OS_Post_Msg_(a, b)
  #define OS_Post_Msg(a, b) (void)OS_Post_Msg_(a, b)
#endif

#define OS_Accept_Msg(a, b)\
{\
  OS_Enter_Critical();\
  *(b) = (a)->message;\
  (a)->message = NULL;\
  OS_Leave_Critical();\
}

#define OS_Accept_MsgI(a, b)\
{\
  *(b) = (a)->message;\
  (a)->message = NULL;\
}

#ifdef OS_OPTI_MSGT
  #define OS_Wait_Msg(a, b)\
  {\
    while ( ( *(b) = OS_Wait_MsgT_(a, 0) ) == NULL ) {\
      OS_Cooperate();\
    }\
  }
#else
  #define OS_Wait_Msg(a, b)\
  {\
    while ( ( *(b) = OS_Wait_Msg_(a) ) == NULL ) {\
      OS_Cooperate();\
    }\
  }
#endif

#define OS_Wait_MsgT(a, b, c)\
{\
  while ( ( *(b) = OS_Wait_MsgT_(a, c) ) == NULL ) {\
    OS_Cooperate();\
    if (OS_Is_TO() ) break;\
  }\
}

#define OS_Read_Msg(a) ( (a)->message )

#define OS_Msg_Create(a, b)\
{\
  (a)->message = (b);\
  (a)->queue = NULL;\
}

/*
SEM
*/
#ifdef OS_USE_BSEM_I
  #define OS_Post_BSemI(a)  OS_Post_Bin_I(a)

  #define OS_Post_BSemR(a, c)\
  {\
    OS_Enter_Critical();\
    *(c) = OS_Post_Bin_I(a);\
    OS_Leave_Critical();\
  }

  #define OS_Post_BSem(a)\
  {\
    OS_Enter_Critical();\
    (void)OS_Post_Bin_I(a);\
    OS_Leave_Critical();\
  }
#else
  #define OS_Post_BSemR(a, c) *(c) = OS_Post_Bin_(a)
  #define OS_Post_BSem(a) (void)OS_Post_Bin_(a)
#endif

#ifdef OS_USE_CSEM_I
  #define OS_Post_CSemI(a, b)  OS_Post_Cnt_I(a, b)
  #define OS_Post_CSemR(a, b, c)\
  {\
    OS_Enter_Critical();\
    *(c) = OS_Post_Cnt_I(a, b);\
    OS_Leave_Critical();\
  }
  #define OS_Post_CSem(a, b)\
  {\
    OS_Enter_Critical();\
    (void)OS_Post_Cnt_I(a, b);\
    OS_Leave_Critical();\
  }
#else
  #define OS_Post_CSemR(a, b, c) *(c) = OS_Post_Cnt_(a, b)
  #define OS_Post_CSem(a, b) (void)OS_Post_Cnt_(a, b)
#endif

#ifdef OS_OPTI_SEMT
  #define OS_Wait_Sem(a)\
  {\
    while ( OS_Wait_SemT_(a, 0) ) {\
      OS_Cooperate();\
    }\
  }
#else
  #define OS_Wait_Sem(a)\
  {\
    while ( OS_Wait_Sem_(a) ) {\
      OS_Cooperate();\
    }\
  }
#endif

#define OS_Wait_SemT(a, b)\
{\
  while ( OS_Wait_SemT_(a, b) ) {\
    OS_Cooperate();\
    if (OS_Is_TO() ) break;\
  }\
}

#define OS_Accept_Sem(a)\
{\
  OS_Enter_Critical();\
  if ( (a)->semaphore != 0) (a)->semaphore--;\
  OS_Leave_Critical();\
}

#define OS_Accept_SemI(a)\
{\
  if ( (a)->semaphore != 0) (a)->semaphore--;\
}

#define OS_Read_Sem(a) ( (a)->semaphore )

#define OS_Sem_Create(a, b)\
{\
  (a)->semaphore = (b);\
  (a)->queue = NULL;\
}

/*
MSGQ
*/
#ifndef OS_USE_MSGQ_I
  #define OS_Post_MsgQR(a, b, c) *(c) = OS_Post_MsgQ_(a, b)
  #define OS_Post_MsgQ(a, b) (void)OS_Post_MsgQ_(a, b)

  #define OS_Accept_MsgQI(a, b) *(b) = OS_Accept_MsgQ_I( (a)->msgqueue )

  #define OS_Accept_MsgQ(a, b)\
  {\
    OS_Enter_Critical();\
    *(b) = OS_Accept_MsgQ_( (a)->msgqueue );\
    OS_Leave_Critical();\
  }
#else
  #define OS_Post_MsgQR(a, b, c)\
  {\
    OS_Enter_Critical();\
    *(c) = OS_Post_MsgQ_I(a, b);\
    OS_Leave_Critical();\
  }

  #define OS_Post_MsgQ(a, b)\
  {\
    OS_Enter_Critical();\
    (void)OS_Post_MsgQ_I(a, b);\
    OS_Leave_Critical();\
  }

  #define OS_Post_MsgQI(a, b) OS_Post_MsgQ_I(a, b)

  #define OS_Accept_MsgQ(a, b)\
  {\
    OS_Enter_Critical();\
    *(b) = OS_Accept_MsgQ_I( (a)->msgqueue );\
    OS_Leave_Critical();\
  }
#endif


#ifdef OS_OPTI_MSGQT
  #define OS_Wait_MsgQ(a, b)\
  {\
    while ( ( *(b) = OS_Wait_MsgQT_(a, 0) ) == NULL ) {\
      OS_Cooperate();\
    }\
  }
#else
  #define OS_Wait_MsgQ(a, b)\
  {\
    while ( ( *(b) = OS_Wait_MsgQ_(a) ) == NULL ) {\
      OS_Cooperate();\
    }\
  }
#endif

#define OS_Wait_MsgQT(a, b, c)\
{\
  while ( ( *(b) = OS_Wait_MsgQT_(a, c) ) == NULL ) {\
    OS_Cooperate();\
    if (OS_Is_TO() ) break;\
  }\
}

#define OS_MsgQ_Create(a, b, c, d)\
{\
  (a)->msgqueue = (b);\
  (a)->queue  = NULL;\
  (b)->head   = (b)->tail = (b)->loaded = 0;\
  (b)->buffer = (OST_MSG_P *)(d);\
  (b)->size   = (c);\
}

/*
FLAGS
*/
#ifdef OS_USE_FLAGS_I
  #define OS_Set_FlagI(a, b) OS_Post_Flag_I(a, b)
  #define OS_Set_Flag(a, b)\
  {\
    OS_Enter_Critical();\
    OS_Post_Flag_I(a, b);\
    OS_Leave_Critical();\
  }
#else
  #define OS_Set_Flag(a, b)  OS_Post_Flag_(a, b)
#endif

#define OS_Wait_Flag(a, b)\
{\
  while ( OS_Wait_Flag_(a, b) ) {\
    OS_Cooperate();\
  }\
}

#define OS_Read_Flag(a) ( (a)->flags )

#define OS_Clear_Flag(a, b)\
{\
  OS_Enter_Critical();\
  (a)->flags &= ~b;\
  OS_Leave_Critical();\
}

#define OS_Clear_FlagI(a, b)\
{\
  (a)->flags &= ~b;\
}

#define OS_Flag_Create(a, b)\
{\
  (a)->flags = (b);\
  (a)->queue = NULL;\
}

/*
TASK
*/
#ifndef OS_Cooperate
  #define OS_Cooperate()\
  {\
    OS_Cooperate_();\
  }
#endif

#ifndef OS_Task_Create
  #if defined(OS_USE_PRIO) && defined(OST_WORKSPACE)
    #define OS_Task_Create(a, b, c, d)\
    {\
      OS_Tree_Rest(a);\
      OS_Task_Init_(a, (OST_TASK_P)b, c, d);\
    }
  #elif defined(OS_USE_PRIO) || defined(OST_WORKSPACE)
    #define OS_Task_Create(a, b, c)\
    {\
      OS_Tree_Rest(a);\
      OS_Task_Init_(a, (OST_TASK_P)b, c);\
    }
  #else
    #define OS_Task_Create(a, b)\
    {\
      OS_Tree_Rest(a);\
      OS_Task_Init_(a, (OST_TASK_P)b);\
    }
  #endif
#endif

#define OS_Task_Stop()\
{\
  OS_Enter_Critical();\
  os_status.state_parts.wait_stop = TRUE;\
  OS_Leave_Critical();\
  OS_Cooperate();\
}

#define OS_Cur_Task() (OST_TASK_P)os_cur_task

#define OS_Task_Status(a) (a)->status.state

#define OS_Task_Resume(a) OS_Task_Resume_( (OST_TASK_P)(a) )

#define OS_Task_Restart(a, b)\
{\
  (b)->entry = (OST_ADDRESS)a;\
}

/*
MISC
*/
#ifndef OS_Scheduler
  #define OS_Scheduler()\
  {\
    OS_Kernel();\
  }
#endif

#ifndef OS_Task_Switch
  #define OS_Task_Switch()\
  {\
    ( (void (*)(void) )(os_cur_task->entry) )();\
  }
#endif

#define OS_Status() (os_status.state)

#define OS_Is_Idle() (os_cur_task == NULL)

#define OS_Delay(a)\
{\
  OS_Delay_Prep(a);\
  OS_Cooperate();\
}

#if defined(OS_OPTI_DELAY) || defined(OS_PROTECT_STAT)
  #define OS_Delay_Prep(a) OS_Delay_Prep_(a)
#else
  #define OS_Delay_Prep(a)\
    OS_Enter_Critical();\
    os_timer_delay = a;\
    os_status.state_parts.wait_timer = TRUE
#endif

#ifndef  OS_Timer
  #ifndef OS_USE_NO_INTER
    #define OS_Timer() if (os_timer_queue != NULL) {os_ticks++;}
  #else
    #define OS_Timer() if (os_timer_queue != NULL) {os_status.state_parts.timer = TRUE;}
  #endif
#endif

#define OS_Timer_Queue() ( (OST_TASK_P)os_timer_queue )

#define OS_Ticks()  (os_ticks)

#define OS_Cur_Delay (os_timer_queue->delay)

/*-----------------------------------------------------------------------*/

typedef union OST_TASK_STS_ {
  OST_U8 state;
  struct {
    OST_BITS    prio    :4;
    OST_BITS    timer   :1;
    OST_BITS    event   :1;
    OST_BITS    stop    :1;
    OST_BITS    timeout :1;
  } state_parts;
} OST_TASK_STS ;

typedef union OST_STATUS_ {
  OST_U8 state;
  struct {
    OST_BITS    timer      :1;
    OST_BITS    event      :1;
    OST_BITS    idle       :1;
    OST_BITS    timeout    :1;
    OST_BITS    wait_timer :1;
    OST_BITS    wait_event :1;
    OST_BITS    wait_stop  :1;
    OST_BITS    wait_prio  :1;
  } state_parts;
} OST_STATUS;

typedef void OS_MSG_CVALIF *OST_MSG_P;
typedef void (__task *OST_FUNC)(void);
typedef struct OST_TASK_D OS_VAR_LOC OS_CVALIF *OST_TASK_P; //

typedef OS_CVALIF struct OST_EVENTS_ {
  OST_TASK_P    queue;
  #ifndef OS_USE_PRIO
  OST_TASK_P    last;
  #endif
} OST_EVENTS;

typedef OST_EVENTS OS_VAR_LOC *OST_EVENT_P;

typedef OS_CVALIF struct OST_TASK_D {
  OST_TASK_P    next;
  OST_ADDRESS   entry;
  #ifdef OS_USE_WORKSPACE
  OST_MSG_P     workspace;
  #endif
  #ifdef OS_USE_EXTENSION
  OST_EXTENS    extension[OST_EXTENS_NO];
  #endif
  #if defined(OS_USE_PRIO) || defined(OS_USE_TASK_STATUS)
  OST_TASK_STS  status;
  #endif
  #ifdef OS_USE_TIMEOUT
  OST_TASK_P    queue;
  OST_EVENT_P   event;
  #endif
  #ifdef OS_USE_DELAY
  OST_TIMER     delay;
  #endif
} OST_TASK_T;

typedef OS_CVALIF struct OST_TASK_N { //
  OST_TASK_P    next;
  OST_ADDRESS   entry;
  #ifdef OS_USE_WORKSPACE
  OST_MSG_P     workspace;
  #endif
  #ifdef OS_USE_EXTENSION
  OST_EXTENS    extension[OST_EXTENS_NO];
  #endif
  #if defined(OS_USE_PRIO) || defined(OS_USE_TASK_STATUS)
  OST_TASK_STS  status;
  #endif
  #ifdef OS_USE_TIMEOUT
  OST_TASK_P    queue;
  #endif
} OST_TASK;

typedef OS_CVALIF struct OST_MESSAGE_ {
  OST_TASK_P    queue;
  #ifndef OS_USE_PRIO
  OST_TASK_P    last;
  #endif
  OST_MSG_P     message;
} OST_MESSAGE;

typedef OST_MESSAGE OS_VAR_LOC *OST_MESSAGE_P;

typedef struct OST_MSGQ_DESCR_ {
  OST_U8    head;
  OST_U8    tail;
  OST_U8    size;
  OST_U8    loaded;
  OST_MSG_P *buffer;
} OST_MSGQ_DESCR;

typedef OST_MSGQ_DESCR OS_VAR_LOC *OST_MSGQ_DESCR_P;

typedef OS_CVALIF struct OST_MSGQUEUE_ {
  OST_TASK_P    queue;
  #ifndef OS_USE_PRIO
  OST_TASK_P    last;
  #endif
  OST_MSGQ_DESCR_P msgqueue;
} OST_MSGQUEUE;

typedef OST_MSGQUEUE OS_VAR_LOC *OST_MSGQUEUE_P;

typedef OS_CVALIF struct OST_SEMAPHORE_ {
  OST_TASK_P    queue;
  #ifndef OS_USE_PRIO
  OST_TASK_P    last;
  #endif
  OST_SEM       semaphore;
} OST_SEMAPHORE;

typedef OST_SEMAPHORE OS_VAR_LOC *OST_SEMAPHORE_P;

typedef OS_CVALIF struct OST_FLAGS_ {
  OST_TASK_P    queue;
  #ifndef OS_USE_PRIO
  OST_TASK_P    last;
  #endif
  OST_FLG       flags;
} OST_FLAGS;

typedef OST_FLAGS OS_VAR_LOC *OST_FLAGS_P;

/*-----------------------------------------------------------------------*/

OS_EXTERN OS_VAR_LOC OS_CVALIF OST_TASK_P os_cur_task;
OS_EXTERN OS_VAR_LOC OS_CVALIF OST_TASK_P os_priv_task;
OS_EXTERN OS_VAR_LOC OS_CVALIF OST_TASK_P os_ready_queue;

#ifdef OS_USE_STACK_SAVE
  OS_EXTERN volatile OST_SP os_sp;
#endif

#ifndef _AVR_REG_
  OS_EXTERN volatile OS_VAR_LOC OS_CVALIF OST_STATUS os_status;
#else
  __regvar __no_init OST_STATUS os_status @ 15;
#endif

#ifdef OS_USE_DELAY
  OS_EXTERN OS_VAR_LOC OS_CVALIF OST_TASK_P os_timer_queue;
  #ifndef OS_USE_NO_INTER
  OS_EXTERN volatile OS_VAR_LOC OS_CVALIF OST_U8 os_ticks;
  #endif
  #ifndef _AVR_REG_
    OS_EXTERN OS_VAR_LOC OS_CVALIF OST_TIMER os_timer_delay;
  #else
    __regvar __no_init OST_TIMER os_timer_delay @ 12;
  #endif
#endif

#if defined(OS_USE_DELAY) || defined(OS_USE_EVENTS) || defined(OSM_PIC_TBL)
  #if !defined(_AVR_REG_) || (OS_DELAY_SIZE >= 2)
    OS_EXTERN OS_VAR_LOC OS_CVALIF OST_EVENT_P os_event_wait;
  #else
    __regvar __no_init OST_EVENT_P os_event_wait @ 13;
  #endif
#endif

#ifdef OS_USE_PRIO
  #ifdef OS_MAIN
  OS_VAR_LOC OS_CVALIF OST_TASK_P os_prio_tbl[OS_MAX_PRIO+1];
  #else
  OS_EXTERN OS_VAR_LOC OS_CVALIF OST_TASK_P os_prio_tbl[];
  #endif
  #ifndef _AVR_REG_
    OS_EXTERN OS_VAR_LOC OS_CVALIF OST_U8 os_cur_prio;
  #else
    __regvar __no_init OST_U8 os_cur_prio @ 14;
  #endif
#endif

#ifdef OS_USE_WORKSPACE
  OS_EXTERN  OST_WORKSPACE workspace;
#endif

/*-----------------------------------------------------------------------*/

OS_CALL_COND void OS_Kernel(void);

OS_CALL_COND void OS_Task_Init_(OST_ADDRESS, OST_TASK_P
    #ifdef OS_USE_PRIO
      , OST_U8
    #endif
    #ifdef OST_WORKSPACE
      , OST_WORKSPACE
    #endif
    );

OS_CALL_COND OST_TASK_P OS_Task_Ins(OST_TASK_P);

OS_CALL_COND void OS_Ready_Qu(OST_EVENT_P);

OS_CALL_COND void OS_Task_Ready_Qu(OST_TASK_P);

OS_CALL_COND void OS_Post_Flag_(OST_FLAGS_P, OST_SEM);

OS_CALL_COND OST_WAIT_MSG OS_Wait_Flag_(OST_FLAGS_P, OST_SEM);

OS_CALL_COND void OS_Post_Flag_I(OST_FLAGS_P, OST_SEM);

OS_CALL_COND OST_ERR_MSG OS_Post_Cnt_(OST_SEMAPHORE_P, OST_SEM );

OS_CALL_COND OST_ERR_MSG OS_Post_Cnt_I(OST_SEMAPHORE_P, OST_SEM );

OS_CALL_COND OST_ERR_MSG OS_Post_Bin_(OST_SEMAPHORE_P);

OS_CALL_COND OST_ERR_MSG OS_Post_Bin_I(OST_SEMAPHORE_P);

OS_CALL_COND OST_WAIT_MSG OS_Wait_Sem_(OST_SEMAPHORE_P);

OS_CALL_COND OST_WAIT_MSG OS_Wait_SemT_(OST_SEMAPHORE_P, OST_TIMER);

OS_CALL_COND OST_ERR_MSG OS_Post_Msg_(OST_MESSAGE_P, OST_MSG_P);

OS_CALL_COND OST_ERR_MSG OS_Post_Msg_I(OST_MESSAGE_P, OST_MSG_P);

OS_CALL_COND OST_MSG_P OS_Wait_Msg_(OST_MESSAGE_P);

OS_CALL_COND OST_MSG_P OS_Wait_MsgT_(OST_MESSAGE_P, OST_TIMER);

OS_CALL_COND OST_ERR_MSG OS_Post_MsgQ_(OST_MSGQUEUE_P, OST_MSG_P);

OS_CALL_COND OST_ERR_MSG OS_Post_MsgQ_I(OST_MSGQUEUE_P, OST_MSG_P);

OS_CALL_COND OST_MSG_P OS_Accept_MsgQ_(OST_MSGQ_DESCR_P);

OS_CALL_COND OST_MSG_P OS_Accept_MsgQ_I(OST_MSGQ_DESCR_P);

OS_CALL_COND OST_MSG_P OS_Wait_MsgQ_(OST_MSGQUEUE_P);

OS_CALL_COND OST_MSG_P OS_Wait_MsgQT_(OST_MSGQUEUE_P, OST_TIMER);

OS_CALL_COND void OS_Wait_TO(OST_EVENT_P, OST_TIMER);

OS_CALL_COND OST_U8 OS_Is_TO(void);

OS_CALL_COND OST_U8 OS_Timeout(void);

OS_CALL_COND void OS_Idle(void);

OS_CALL_COND OST_ERR_MSG OS_Task_Resume_(OST_TASK_P);

OS_CALL_COND OST_ERR_MSG OS_Task_Resume_I(OST_TASK_P);

OS_CALL_COND void OS_Sw_Hook(void);

OS_CALL_COND void OS_Delay_Prep_(OST_TIMER);

#ifndef OS_DECL_REM
  OS_CALL_COND void OS_Cooperate_(OS_COOP_PARAMT);
#endif

OS_CALL_COND void OS_Dis_Int_(void);

OS_CALL_COND void OS_Ena_Int_(void);

OS_CALL_COND void OS_Init(void);

#endif //_JACOS_H

